import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:get/src/routes/observers/route_observer.dart';

import 'get_main.dart';

class GetMaterialApp extends StatelessWidget {
  const GetMaterialApp({
    Key key,
    this.navigatorKey,
    this.home,
    this.routes = const <String, GetRoute>{},
    this.initialRoute,
    this.onGenerateRoute,
    this.onGenerateInitialRoutes,
    this.onUnknownRoute,
    this.navigatorObservers = const <NavigatorObserver>[],
    this.builder,
    this.title = '',
    this.onGenerateTitle,
    this.color,
    this.theme,
    this.darkTheme,
    this.themeMode = ThemeMode.system,
    this.locale,
    this.localizationsDelegates,
    this.localeListResolutionCallback,
    this.localeResolutionCallback,
    this.supportedLocales = const <Locale>[Locale('en', 'US')],
    this.debugShowMaterialGrid = false,
    this.showPerformanceOverlay = false,
    this.checkerboardRasterCacheImages = false,
    this.checkerboardOffscreenLayers = false,
    this.showSemanticsDebugger = false,
    this.debugShowCheckedModeBanner = true,
    this.shortcuts,
    this.routingCallback,
    this.defaultTransition,
    this.actions,
    this.opaqueRoute,
    this.enableLog,
    this.popGesture,
    this.transitionDuration,
    this.defaultGlobalState,
    this.namedRoutes,
    this.unknownRoute,
  })  : assert(routes != null),
        assert(navigatorObservers != null),
        assert(title != null),
        assert(debugShowMaterialGrid != null),
        assert(showPerformanceOverlay != null),
        assert(checkerboardRasterCacheImages != null),
        assert(checkerboardOffscreenLayers != null),
        assert(showSemanticsDebugger != null),
        assert(debugShowCheckedModeBanner != null),
        super(key: key);

  final GlobalKey<NavigatorState> navigatorKey;

  final Widget home;

  final Map<String, GetRoute> routes;

  final String initialRoute;

  final RouteFactory onGenerateRoute;

  final InitialRouteListFactory onGenerateInitialRoutes;

  final RouteFactory onUnknownRoute;

  final List<NavigatorObserver> navigatorObservers;

  final TransitionBuilder builder;

  final String title;

  final GenerateAppTitle onGenerateTitle;

  final ThemeData theme;

  final ThemeData darkTheme;

  final ThemeMode themeMode;

  final Color color;

  final Locale locale;

  final Iterable<LocalizationsDelegate<dynamic>> localizationsDelegates;

  final LocaleListResolutionCallback localeListResolutionCallback;

  final LocaleResolutionCallback localeResolutionCallback;

  final Iterable<Locale> supportedLocales;

  final bool showPerformanceOverlay;

  final bool checkerboardRasterCacheImages;

  final bool checkerboardOffscreenLayers;

  final bool showSemanticsDebugger;

  final bool debugShowCheckedModeBanner;

  final Map<LogicalKeySet, Intent> shortcuts;

  final Map<LocalKey, ActionFactory> actions;

  final bool debugShowMaterialGrid;

  final Function(Routing) routingCallback;
  final Transition defaultTransition;
  final bool opaqueRoute;
  final bool enableLog;
  final bool popGesture;
  final Duration transitionDuration;
  final bool defaultGlobalState;
  final Map<String, GetRoute> namedRoutes;
  final GetRoute unknownRoute;

  Route<dynamic> namedRoutesGenerate(RouteSettings settings) {
    Get.setSettings(settings);
    if (namedRoutes.containsKey(settings.name)) {
      return namedRoutes[settings.name];
    } else {
      return unknownRoute;
    }
  }

  @override
  Widget build(BuildContext context) {
    Get.config(
      enableLog: enableLog ?? Get.isLogEnable,
      defaultTransition: defaultTransition ?? Get.defaultTransition,
      defaultOpaqueRoute: opaqueRoute ?? Get.isOpaqueRouteDefault,
      defaultPopGesture: popGesture ?? Get.isPopGestureEnable,
      defaultDurationTransition:
          transitionDuration ?? Get.defaultDurationTransition,
      defaultGlobalState: defaultGlobalState ?? Get.defaultGlobalState,
    );

    return MaterialApp(
      navigatorKey: (navigatorKey == null ? Get.key : Get.addKey(navigatorKey)),
      home: home,
      routes: routes ?? const <String, WidgetBuilder>{},
      initialRoute: initialRoute,
      onGenerateRoute:
          (namedRoutes == null ? onGenerateRoute : namedRoutesGenerate),
      onGenerateInitialRoutes: onGenerateInitialRoutes,
      onUnknownRoute: onUnknownRoute,
      navigatorObservers: (navigatorObservers == null
          ? <NavigatorObserver>[GetObserver(routingCallback)]
          : <NavigatorObserver>[GetObserver(routingCallback)]
        ..addAll(navigatorObservers)),
      builder: builder,
      title: title ?? '',
      onGenerateTitle: onGenerateTitle,
      color: color,
      theme: theme,
      darkTheme: darkTheme,
      themeMode: themeMode ?? ThemeMode.system,
      locale: locale,
      localizationsDelegates: localizationsDelegates,
      localeListResolutionCallback: localeListResolutionCallback,
      localeResolutionCallback: localeResolutionCallback,
      supportedLocales: supportedLocales ?? const <Locale>[Locale('en', 'US')],
      debugShowMaterialGrid: debugShowMaterialGrid ?? false,
      showPerformanceOverlay: showPerformanceOverlay ?? false,
      checkerboardRasterCacheImages: checkerboardRasterCacheImages ?? false,
      checkerboardOffscreenLayers: checkerboardOffscreenLayers ?? false,
      showSemanticsDebugger: showSemanticsDebugger ?? false,
      debugShowCheckedModeBanner: debugShowCheckedModeBanner ?? true,
      shortcuts: shortcuts,
      actions: actions,
    );
  }
}
